package ptglob

import (
	"fmt"
	"testing"
)

func testM1(t *testing.T, text, pattern string, want bool) {
	res, matches := MatchParts(pattern, text)
	//	log.Printf("M('%v','%v') => %v %v", text, pattern, res, matches)
	if want != res {
		t.Errorf("Match('%v','%v') should return %v but got %v/%v", text, pattern, want, res, matches)
	}
}

func testR1(t *testing.T, text, pattern, rewrite, want string) {
	ok, res := Rewrite(pattern, text, rewrite)
	//	log.Printf("R('%v','%v','%v') => %v %v", text, pattern, rewrite, ok, res)
	if want == "" {
		if ok {
			t.Errorf("Rewrite('%v','%v','%v') should return false but got true", text, pattern, rewrite)
		}
	} else {
		if !ok {
			t.Errorf("Rewrite('%v','%v','%v') should return true but got false", text, pattern, rewrite)
		} else {
			if res != want {
				t.Errorf("Rewrite('%v','%v','%v') should return '%v' but got '%v'", text, pattern, rewrite, want, res)
			}
		}
	}
}

func TestPTGlob(t *testing.T) {
	testM1(t, "a/b", "*/b", true)
	testM1(t, "a/b", "*b", true)
	testM1(t, "ab", "*b", true)
	testM1(t, "ab", "a*", true)
	testM1(t, "", "", true)
	testM1(t, "x", "", false)
	testM1(t, "", "x", false)
	testM1(t, "a", "*", true)
	testM1(t, "a", "*a", true)
	testM1(t, "a", "*b", false)
	testM1(t, "abc", "*b*", true)
	testM1(t, "abc", "*bc", true)
	testM1(t, "abc", "ab*", true)
	testM1(t, "abcdef", "*ab*ef", true)
	testM1(t, "abcdef", "*ab*ef", true)
	testM1(t, "abcdef", "*ab?ef", false)
	testM1(t, "abcdef", "*ab??ef", true)
	testM1(t, "abcdef", "*c*e*", true)
	testM1(t, "abcdefg", "*c*e*g", true)
	testR1(t, "abcdefg", "ab*e*g", "x%1", "xcd")
	testR1(t, "abcdefg", "ab*e*g", "%%1", "%1")
	testR1(t, "abcdefg", "ab*e*g", ".%1.%1.%2.", ".cd.cd.f.")
}

func ExampleMatch() {
	fmt.Printf("%v\n", Match("*", "abc"))
	fmt.Printf("%v\n", Match("a*", "abc"))
	fmt.Printf("%v\n", Match("b*", "abc"))
	fmt.Println(MatchParts("a*", "abc"))
	// Output: true
	// true
	// false
	// true [bc]
}
